package boltease

import (
	"reflect"
)

func (b *DB) LoadStruct(path []string, dest interface{}) error {
	return nil
}
func (b *DB) SaveStruct(path []string, src interface{}) error {
	t := reflect.TypeOf(src)
	fields := reflect.VisibleFields(t)
	// Build the PropertyList
	var props PropertyList
	for _, fld := range fields {
		if FieldIgnored(fld) {
			continue
		}
		p := Property{
			Path:  path,
			Name:  FieldName(fld),
			Value: reflect.ValueOf(fld),
		}
		props = append(props, p)
	}
	return b.SavePropertyList(path, &props)
}

type Property struct {
	Path  []string
	Name  string
	Value reflect.Value
}

func PropertyValueToInterface(val reflect.Value) interface{} {
	if val.CanInterface() {
		return val.Interface()
	}
	switch val.Kind() {
	case reflect.Int, reflect.Int8, reflect.Int16, reflect.Int32, reflect.Int64:
		return val.Int()
	case reflect.Bool:
		return val.Bool()
	case reflect.String:
		return val.String()
	default:
		return val.Bytes()
	}
}

func FieldName(fld reflect.StructField) string {
	nm := fld.Name
	tag := fld.Tag.Get("boltease")
	if tag != "" {
		nm = tag
	}
	return nm
}
func FieldIgnored(fld reflect.StructField) bool {
	return fld.Tag.Get("boltease") == "-"
}

type PropertyList []Property

func (b *DB) LoadPropertyList(path []string) (*PropertyList, error) {
	return nil, nil
}
func (b *DB) SavePropertyList(path []string, l *PropertyList) error {
	// Make sure the path exists
	err := b.MkBucketPath(path)
	if err != nil {
		return err
	}
	for _, prop := range *l {
		propPath := append(path, prop.Path...)
		err = b.MkBucketPath(propPath)
		if err != nil {
			return err
		}
		b.Set(propPath, prop.Name, prop.Value)
	}
	return nil
}
